﻿module libbz2.bzlib;
private import std.c.stdio;

//----------------------------------------------------------------
// bzlib.d :
//   manually translated from "bzlib.h" to d header module.
//                                       2004/08/19 k.inaba
//----------------------------------------------------------------

//       VVV   original copyright notice of "bzlib.h" VVV       //
/*--
  This file is a part of bzip2 and/or libbzip2, a program and
  library for lossless, block-sorting data compression.

  Copyright (C) 1996-2002 Julian R Seward.  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  1. Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

  2. The origin of this software must not be misrepresented; you must 
     not claim that you wrote the original software.  If you use this 
     software in a product, an acknowledgment in the product 
     documentation would be appreciated but is not required.

  3. Altered source versions must be plainly marked as such, and must
     not be misrepresented as being the original software.

  4. The name of the author may not be used to endorse or promote 
     products derived from this software without specific prior written 
     permission.

  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
  GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

  Julian Seward, Cambridge, UK.
  jseward@acm.org
  bzip2/libbzip2 version 1.0 of 21 March 2000

  This program is based on (at least) the work of:
     Mike Burrows
     David Wheeler
     Peter Fenwick
     Alistair Moffat
     Radford Neal
     Ian H. Witten
     Robert Sedgewick
     Jon L. Bentley

  For more information on these sources, see the manual.
--*/

enum
{
	BZ_RUN    = 0,
	BZ_FLUSH  = 1,
	BZ_FINISH = 2,
}

enum
{
	BZ_OK                = 0,
	BZ_RUN_OK            = 1,
	BZ_FLUSH_OK          = 2,
	BZ_FINISH_OK         = 3,
	BZ_STREAM_END        = 4,
	BZ_SEQUENCE_ERROR    = (-1),
	BZ_PARAM_ERROR       = (-2),
	BZ_MEM_ERROR         = (-3),
	BZ_DATA_ERROR        = (-4),
	BZ_DATA_ERROR_MAGIC  = (-5),
	BZ_IO_ERROR          = (-6),
	BZ_UNEXPECTED_EOF    = (-7),
	BZ_OUTBUFF_FULL      = (-8),
	BZ_CONFIG_ERROR      = (-9),
}

struct bz_stream
{
    char *next_in;
    uint avail_in;
    uint total_in_lo32;
    uint total_in_hi32;

    char *next_out;
    uint avail_out;
    uint total_out_lo32;
    uint total_out_hi32;

    void *state;

    void* function(void *,int,int) bzalloc;
    void function(void *,void *) bzfree;
    void *opaque;
} 

extern(Windows)
{
/*-- Core (low-level) library functions --*/

	int BZ2_bzCompressInit(
	      bz_stream* strm, 
	      int        blockSize100k, 
	      int        verbosity, 
	      int        workFactor 
	   );

	int BZ2_bzCompress( 
	      bz_stream* strm, 
	      int action 
	   );

	int BZ2_bzCompressEnd( 
	      bz_stream* strm 
	   );

	int BZ2_bzDecompressInit( 
	      bz_stream *strm, 
	      int       verbosity, 
	      int       small
	   );

	int BZ2_bzDecompress( 
	      bz_stream* strm 
	   );

	int BZ2_bzDecompressEnd( 
	      bz_stream *strm 
	   );

/*-- High(er) level library functions --*/

	enum { BZ_MAX_UNUSED = 5000 }
	alias void BZFILE;

	BZFILE* BZ2_bzReadOpen( 
	      int*  bzerror,   
	      FILE* f, 
	      int   verbosity, 
	      int   small,
	      void* unused,    
	      int   nUnused 
	   );

	void BZ2_bzReadClose( 
	      int*    bzerror, 
	      BZFILE* b 
	   );

	void BZ2_bzReadGetUnused( 
	      int*    bzerror, 
	      BZFILE* b, 
	      void**  unused,  
	      int*    nUnused 
	   );

	int BZ2_bzRead( 
	      int*    bzerror, 
	      BZFILE* b, 
	      void*   buf, 
	      int     len 
	   );

	BZFILE* BZ2_bzWriteOpen( 
	      int*  bzerror,      
	      FILE* f, 
	      int   blockSize100k, 
	      int   verbosity, 
	      int   workFactor 
	   );

	void BZ2_bzWrite( 
	      int*    bzerror, 
	      BZFILE* b, 
	      void*   buf, 
	      int     len 
	   );

	void BZ2_bzWriteClose( 
	      int*          bzerror, 
	      BZFILE*       b, 
	      int           abandon, 
	      uint*         nbytes_in, 
	      uint*         nbytes_out 
	   );

	void BZ2_bzWriteClose64( 
	      int*          bzerror, 
	      BZFILE*       b, 
	      int           abandon, 
	      uint*         nbytes_in_lo32, 
	      uint*         nbytes_in_hi32, 
	      uint*         nbytes_out_lo32, 
	      uint*         nbytes_out_hi32
	   );

/*-- Utility functions --*/

	int BZ2_bzBuffToBuffCompress( 
	      char*         dest, 
	      uint*         destLen,
	      char*         source, 
	      uint          sourceLen,
	      int           blockSize100k, 
	      int           verbosity, 
	      int           workFactor 
	   );

	int BZ2_bzBuffToBuffDecompress( 
	      char*         dest, 
	      uint*         destLen,
	      char*         source, 
	      int           sourceLen,
	      int           small, 
	      int           verbosity 
	   );

/*--
   Code contributed by Yoshioka Tsuneo
   (QWF00133@niftyserve.or.jp/tsuneo-y@is.aist-nara.ac.jp),
   to support better zlib compatibility.
   This code is not _officially_ part of libbzip2 (yet);
   I haven't tested it, documented it, or considered the
   threading-safeness of it.
   If this code breaks, please contact both Yoshioka and me.
--*/

	char* BZ2_bzlibVersion(
	   );

	BZFILE * BZ2_bzopen(
	      char *path,
	      char *mode
	   );

	BZFILE * BZ2_bzdopen(
	      int        fd,
	      char *mode
	   );
         
	int BZ2_bzread(
	      BZFILE* b, 
	      void* buf, 
	      int len 
	   );

	int BZ2_bzwrite(
	      BZFILE* b, 
	      void*   buf, 
	      int     len 
	   );

	int BZ2_bzflush(
	      BZFILE* b
	   );

	void BZ2_bzclose(
	      BZFILE* b
	   );

	char * BZ2_bzerror(
	      BZFILE *b, 
	      int    *errnum
	   );
}
